----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:48:25 12/03/2007 
-- Design Name: 
-- Module Name:    automaton - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity automaton is
	
	-- I/O SIGNALI
	port(
		CLK_in	: in STD_LOGIC;
		RST_in	: in STD_LOGIC;
		BTN_S_in	: in STD_LOGIC;
		BTN_N_in	: in STD_LOGIC;
		BTN_E_in	: in STD_LOGIC;
		
		-- izhodi proti LCD-ju
		LCD_E_out	: out	STD_LOGIC;
		LCD_RS_out	: out	STD_LOGIC;
		LCD_RW_out	: out	STD_LOGIC;
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);
		LED			: out STD_LOGIC_VECTOR(3 downto 0)
	);


end automaton;

architecture Behavioral of automaton is

-- COMPONENT DESCRIPTION
	COMPONENT lcd_module
	PORT(
		CLK_in : IN std_logic;
		RST_in : IN std_logic;
		DATA_RDY_in : IN std_logic;
		DATA_TO_SEND_in : IN std_logic_vector(9 downto 0);    
		SF_D_inout : INOUT std_logic_vector(11 downto 8);      
		LCD_READY_out : OUT std_logic;
		LCD_E_out : OUT std_logic;
		LCD_RS_out : OUT std_logic;
		LCD_RW_out : OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT btn_pressed
	PORT(
		CLK_in : IN std_logic;
		RST_in : IN std_logic;
		BTN_in : IN std_logic;          
		SIG_out : OUT std_logic
		);
	END COMPONENT;
	
	
-- INTERNAL SIGNALS
	signal LCD_E	: STD_LOGIC;
	signal LCD_RS	: STD_LOGIC;
	signal LCD_RW	: STD_LOGIC;
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);
	signal DATA_RDY	: STD_LOGIC;
	signal LCD_READY	: STD_LOGIC;
	signal DATA_TO_SEND	:	STD_LOGIC_VECTOR(9 downto 0);
	
	signal BTN_E, BTN_N, BTN_S : STD_LOGIC;
	
-- AUTOMATON STATES
	type states is (idle, clr_scr, send_a, send_b);
	signal currSt, nextSt : states;

begin

-- IO ASSIGNMENT
		
		LCD_E_out	<= LCD_E;
		LCD_RS_out	<= LCD_RS;
		LCD_RW_out	<= LCD_RW;
		SF_D_inout	<= SF_D;

-- COMPONENT INSTANTIATION
	LCD: lcd_module PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		DATA_RDY_in =>	DATA_RDY,
		LCD_READY_out =>	LCD_READY,
		DATA_TO_SEND_in =>	DATA_TO_SEND,
		LCD_E_out =>	LCD_E,
		LCD_RS_out =>	LCD_RS,
		LCD_RW_out =>	LCD_RW,
		SF_D_inout =>	SF_D 
	);
	
	BUTTON_S: btn_pressed PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		BTN_in => BTN_S_in,
		SIG_out => BTN_S
	);
	
	BUTTON_N: btn_pressed PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		BTN_in => BTN_N_in,
		SIG_out => BTN_N
	);
	
	BUTTON_E: btn_pressed PORT MAP(
		CLK_in => CLK_in,
		RST_in => RST_in,
		BTN_in => BTN_E_in,
		SIG_out => BTN_E
	);
	
-- AUTOMATON
	process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if (RST_in = '1') then
				currSt	<= idle;
			else
				currSt	<= nextSt;
			end if;
		end if;	
	end process;

	transition_func: process (currSt, LCD_READY, BTN_S, BTN_N, BTN_E)
	begin
		case currSt is
		-- IDLE
		when idle =>
			if (BTN_S = '1') then
				nextSt	<= send_a;
			elsif (BTN_N = '1') then
				nextSt	<= clr_scr;
			elsif (BTN_E = '1') then
				nextSt	<= send_b;
			else
				nextSt	<= idle;
			end if;
		-- CLR SCR
		when clr_scr =>
			if (LCD_READY = '1') then
				nextSt <= idle;
			else
				nextSt <= clr_scr;
			end if;
		-- SEND A
		when send_a =>
			if (LCD_READY = '1') then
				nextSt	<= idle;
			else
				nextSt <= send_a;
			end if;
		-- SEND B
		when send_b =>
			if (LCD_READY = '1') then
				nextSt	<= idle;
			else
				nextSt <= send_b;
			end if;
		end case;
	end process;
	
	output_func:process (currSt)
	begin
		case currSt is
		-- IDLE
		when idle =>
			DATA_RDY <= '0';
			DATA_TO_SEND <= (others => '0');
			LED	<= "0001";
		
		-- CLR SCR
		when clr_scr =>
			DATA_RDY			<= '1';
			DATA_TO_SEND	<= "0000000001";		
			LED	<= "0010";
		
		-- SEND A
		when send_a =>
			DATA_RDY			<= '1';
			DATA_TO_SEND	<= "1001000001";
			LED	<= "0100";
		
		-- SEND B
		when send_b =>
			DATA_RDY			<= '1';
			DATA_TO_SEND	<= "1001000010";
			LED	<= "1000";
		end case;	
		
	end process;
end Behavioral;

