----------------------------------------------------------------------------------
-- Company: FRI
-- Engineer: Miha Moskon
-- 
-- Create Date:    17:01:12 10/03/2007 
-- Design Name: 
-- Module Name:    lcd_send - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lcd_send is
	port(
		CLK_in	: in	STD_LOGIC;		-- ura
		RST_in	: in	STD_LOGIC;		-- skupni reset signal
		
		LCD_E_out	: out	STD_LOGIC;		-- lcd enable 
		LCD_RS_out	: out	STD_LOGIC;	-- lcd reg. select
		LCD_RW_out	: out	STD_LOGIC;	-- lcd read/write
		SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
		
		DATA_in	:	in STD_LOGIC_VECTOR(9 downto 0);		-- ukaz/podatek, ki ga posiljamo LCD-ju
		
		SEND_IN_PROG_in	: in	STD_LOGIC;	-- odvija se posiljanje
		SEND_DONE_out	: out	STD_LOGIC);		-- posiljanje je koncano
		
end lcd_send;

architecture Behavioral of lcd_send is
	component counter
		generic(
			size : integer := 8);
		port(
			CLK_in	: IN STD_LOGIC;
			WE_in	: IN STD_LOGIC;
			VALUE_in	: IN STD_LOGIC_VECTOR(size-1 downto 0);
			COUNT_UP_out	: OUT STD_LOGIC);		
	end component;
	
	component lcd_send_nibble
		port(
			CLK_in	: in	STD_LOGIC;
			RST_in	: in	STD_LOGIC;
			LCD_E_out	: out	STD_LOGIC;
			LCD_RS_out	: out	STD_LOGIC;	
			LCD_RW_out	: out	STD_LOGIC;	
			SF_D_inout	: inout	STD_LOGIC_VECTOR(11 downto 8);
			DATA_NIBBLE_in	:	in STD_LOGIC_VECTOR(5 downto 0);	
			NIBBLE_SEND_IN_PROG_in	: in	STD_LOGIC;	
			NIBBLE_SEND_DONE_out	: out	STD_LOGIC);	
	end component;

	
	
	-- stanja avtomata za posiljanje
	type STATES is (DUMMY, UPPER_NIBBLE, INIT_DELAY1, DELAY1, LOWER_NIBBLE, INIT_DELAY2, DELAY2, DONE);		-- definiranje moznih stanj avtomata
	signal currSt, nextSt	:	STATES;		-- trenutno in naslednje stanje
	
	-- signali za stevec
	signal WE_counter, count_up	: STD_LOGIC;
	signal value_to_counter	: STD_LOGIC_VECTOR(11 downto 0);	-- znati mora steti do 2.000 (11 bitov)
	
	-- signali za lcd_send_nibble
	signal LCD_E_NIBBLE	: STD_LOGIC;
	signal LCD_RS_NIBBLE	: STD_LOGIC;
	signal LCD_RW_NIBBLE	: STD_LOGIC;
	signal SF_D_NIBBLE	: STD_LOGIC_VECTOR(11 downto 8);
	signal DATA_NIBBLE	: STD_LOGIC_VECTOR(5 downto 0);
	signal nibble_send_in_prog	:	STD_LOGIC;
	signal nibble_send_done	: STD_LOGIC;
	
	-- signali, ki jih povezemo z izhodi
	signal LCD_E	: STD_LOGIC;		-- lcd enable 
	signal LCD_RS	: STD_LOGIC;	-- lcd reg. select
	signal LCD_RW	: STD_LOGIC;	-- lcd read/write
	signal SF_D	: STD_LOGIC_VECTOR(11 downto 8);	-- podatkovni signali LCD-ja
	signal SEND_DONE	: STD_LOGIC;		-- posiljanje je koncano
	
begin

	-- IO ASSIGNMENT
	LCD_E_out	<= LCD_E;
	LCD_RS_out	<= LCD_RS;
	LCD_RW_out	<= LCD_RW;
	SF_D_inout	<= SF_D;
	SEND_DONE_out	<= SEND_DONE;
	


--------------------------
-- INSTANCIRANJE KOMPONENT
--------------------------

	-- instanca stevca
	counter11bit:counter
		generic map
		( 
			size => 12		-- 11-bitni stevec: znati mora steti do 2.000
		)
		port map
		(
			CLK_in	=> CLK_in,
			WE_in	=> WE_counter,
			VALUE_in	=> value_to_counter,
			COUNT_UP_out	=> count_up		
		);
		
	-- instanca lcd_send_nibble
	nibble_send:lcd_send_nibble
		port map
		(
			CLK_in	=> CLK_in,
			RST_in	=> RST_in,
			LCD_E_out =>	LCD_E_NIBBLE,
			LCD_RS_out	=> LCD_RS_NIBBLE,
			LCD_RW_out	=>	LCD_RW_NIBBLE,
			SF_D_inout	=> SF_D_NIBBLE,
			DATA_NIBBLE_in	=> DATA_NIBBLE,
			NIBBLE_SEND_IN_PROG_in	=> nibble_send_in_prog,
			NIBBLE_SEND_DONE_out	=> nibble_send_done
		);

--------------------
-- SEND DATA AVTOMAT
--------------------

	sync_proc:process (CLK_in)
	begin
		if (rising_edge(CLK_in)) then
			if ( (RST_in = '1') OR (SEND_IN_PROG_in = '0') ) then 
				currSt	<= DUMMY;
			else
				currSt	<= nextSt;
			end if;
		end if;
	end process;
	
	send_proc: process (currSt, SEND_IN_PROG_in, count_up, nibble_send_done)
	begin
		-- lcd		
		LCD_E	<= '0';
		LCD_RS	<= '0';
		LCD_RW	<= '0';
		SF_D	<= "0000";
		
		-- stevec
		WE_counter <= '0';
		value_to_counter <= (others => '0');
		
		-- vhodi v lcd_top
		SEND_DONE <= '0';
		
		-- vhodi v nibble
		DATA_NIBBLE	<=	(others => '0');
		nibble_send_in_prog	<= '0';
		
		case currSt is
			
			-- cakanje na pripravljen podatek
			when DUMMY	=>
				if (SEND_IN_PROG_in = '0') then
					nextSt	<= DUMMY;
				else
					nextSt	<= UPPER_NIBBLE;
				end if;
				
			-- posiljanje zgornjega dela podatkov
			when UPPER_NIBBLE	=>
				
				-- vhodi v nibble
				DATA_NIBBLE	<=	DATA_in(9 downto 4);	-- zgornji del podatkov
				nibble_send_in_prog	<= '1';
				
				-- izhodi iz nibbla
				LCD_E	<= LCD_E_NIBBLE;
				LCD_RS	<=	LCD_RS_NIBBLE;
				LCD_RW	<=	LCD_RW_NIBBLE;
				SF_D	<= SF_D_NIBBLE;
				
				-- v naslednje stanje ne grem, dokler ni prenos nibbla koncan
				if (nibble_send_done = '0')	then
					nextSt	<= UPPER_NIBBLE;
				else
					nextSt	<= INIT_DELAY1;
				end if;
			
			-- inicializacija zakasnitve med posiljanjem zgornjega in spodnjega dela podatka
			when INIT_DELAY1	=>
				WE_counter	<= '1';
				value_to_counter	<= x"032";		-- 50 u.p. (1us)
				
				nextSt	<= DELAY1;
				
			-- zakasnitev med posiljanjem zgornjega in spodnjega dela podatka				
			when DELAY1	=>
				if (count_up = '0') then
					nextSt	<= DELAY1;
				else
					nextSt	<= LOWER_NIBBLE;
				end if;
				
			-- posiljanje spodnjega dela podatkov
			when LOWER_NIBBLE =>
				-- vhodi v nibble
				DATA_NIBBLE	<=	DATA_in(9 downto 8) & DATA_in(3 downto 0);	-- spodnji del podatkov
				nibble_send_in_prog	<= '1';
				
				-- izhodi iz nibbla
				LCD_E	<= LCD_E_NIBBLE;
				LCD_RS	<=	LCD_RS_NIBBLE;
				LCD_RW	<=	LCD_RW_NIBBLE;
				SF_D	<= SF_D_NIBBLE;
				
				-- v naslednje stanje ne grem, dokler ni prenos nibbla koncan
				if (nibble_send_done = '0')	then
					nextSt	<= LOWER_NIBBLE;
				else
					nextSt	<= INIT_DELAY2;
				end if;
				
			-- inicializacija zakasnitve za posiljanjem podatka
			when INIT_DELAY2	=>
				WE_counter	<= '1';
				value_to_counter	<=	x"7D0";		-- 2000 u.p. (40us)
				
				nextSt	<= DELAY2;
				
			-- zakasnitev za posiljanjem podatka
			when DELAY2	=>
				if (count_up = '0') then
					nextSt	<= DELAY2;
				else
					nextSt	<= DONE;
				end if;
			
			-- prenos koncan
			when	DONE =>
				SEND_DONE <= '1';
				
				nextSt	<= DUMMY;
			
		end case;

	end process;
	
end Behavioral;

